@@ -134,14 +134,17 @@ bool dpx::Writer::WriteHeader()
 
 void dpx::Writer::SetUserData(const long size)
 {
-	// TODO
+	this->header.SetUserSize(size);
 }
 
 
 bool dpx::Writer::WriteUserData(void *data)
 {
-	// XXX TODO
-	return false;
+    size_t size = this->header.UserSize();
+	if (fd->Write(data, size) != size)
+		return false;
+    this->fileLoc += size;
+	return true;
 }
 
 
@@ -175,6 +178,19 @@ void dpx::Writer::SetElement(const int num, const Descriptor desc, const U8 bitD
 	this->header.CalculateNumberOfElements();
 }
 
+bool
+dpx::Writer::WritePadData(const int alignment)
+{
+    int imageoffset = ((this->fileLoc + alignment - 1)/alignment)*alignment;
+    int padsize = imageoffset - this->fileLoc;
+    if (padsize > 0) {
+        std::vector<dpx::U8> pad (padsize, 0xff);
+        this->fileLoc += this->fd->Write (&pad[0], padsize);
+        if (this->fileLoc != imageoffset)
+            return false;
+    }
+    return true;
+}
 
 // the data is processed so write it straight through
 // argument count is total size in bytes of the passed data
@@ -188,6 +204,10 @@ bool dpx::Writer::WriteElement(const int element, void *data, const long count)
 	if (this->header.ImageDescriptor(element) == kUndefinedDescriptor)
 		return false;
 
+    // The DPX spec recommends that the image data starts on a 8K boundry.
+    if (! this->WritePadData(0x2000))
+        return false;
+
 	// update file ptr
 	this->header.SetDataOffset(element, this->fileLoc);
 	this->fileLoc += count;
@@ -211,8 +231,6 @@ bool dpx::Writer::WriteElement(const int element, void *data)
 	return this->WriteElement(element, data, this->header.ComponentDataSize(element));
 }
 
-
-
 bool dpx::Writer::WriteElement(const int element, void *data, const DataSize size)
 {
 	bool status = true;	
@@ -225,6 +243,10 @@ bool dpx::Writer::WriteElement(const int element, void *data, const DataSize siz
 	if (this->header.ImageDescriptor(element) == kUndefinedDescriptor)
 		return false;
 
+    // The DPX spec recommends that the image data starts on a 8K boundry.
+    if (! this->WritePadData(0x2000))
+        return false;
+
 	// mark location in headers
 	if (element == 0)
 		this->header.SetImageOffset(this->fileLoc);
