@@ -36,12 +36,12 @@
 #include "ico.h"
 #include "../png.imageio/png_pvt.h"
 
-#include "dassert.h"
-#include "typedesc.h"
-#include "imageio.h"
-#include "thread.h"
-#include "strutil.h"
-#include "fmath.h"
+#include "OpenImageIO/dassert.h"
+#include "OpenImageIO/typedesc.h"
+#include "OpenImageIO/imageio.h"
+#include "OpenImageIO/thread.h"
+#include "OpenImageIO/strutil.h"
+#include "OpenImageIO/fmath.h"
 
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
@@ -58,6 +58,9 @@ public:
     virtual bool close ();
     virtual bool write_scanline (int y, int z, TypeDesc format,
                                  const void *data, stride_t xstride);
+    virtual bool write_tile (int x, int y, int z, TypeDesc format,
+                             const void *data, stride_t xstride,
+                             stride_t ystride, stride_t zstride);
 
 private:
     std::string m_filename;           ///< Stash the filename
@@ -69,6 +72,8 @@ private:
     int m_xor_slb;                    ///< XOR mask scanline length in bytes
     int m_and_slb;                    ///< AND mask scanline length in bytes
     int m_bpp;                        ///< Bits per pixel
+    unsigned int m_dither;
+    std::vector<unsigned char> m_tilebuffer;
 
     png_structp m_png;                ///< PNG read structure pointer
     png_infop m_info;                 ///< PNG image info structure pointer
@@ -213,6 +218,9 @@ ICOOutput::open (const std::string &name, const ImageSpec &userspec,
             m_spec.set_format (TypeDesc::UINT8);
     }
 
+    m_dither = (m_spec.format == TypeDesc::UINT8) ?
+                    m_spec.get_int_attribute ("oiio:dither", 0) : 0;
+
     //std::cerr << "[ico] writing at " << m_bpp << "bpp\n";
 
     m_file = Filesystem::fopen (name, mode == AppendSubimage ? "r+b" : "wb");
@@ -347,7 +355,6 @@ ICOOutput::open (const std::string &name, const ImageSpec &userspec,
         return false;
     }
 
-
     fseek (m_file, m_offset, SEEK_SET);
     if (m_want_png) {
         // unused still, should do conversion to unassociated
@@ -394,6 +401,11 @@ ICOOutput::open (const std::string &name, const ImageSpec &userspec,
         fseek (m_file, m_offset + sizeof (bmi), SEEK_SET);
     }
 
+    // If user asked for tiles -- which this format doesn't support, emulate
+    // it by buffering the whole image.
+    if (m_spec.tile_width && m_spec.tile_height)
+        m_tilebuffer.resize (m_spec.image_bytes());
+
     return true;
 }
 
@@ -413,19 +425,28 @@ ICOOutput::supports (const std::string &feature) const
 bool
 ICOOutput::close ()
 {
-    //std::cerr << "[ico] closing\n";
+    if (! m_file) {   // already closed
+        init ();
+        return true;
+    }
+
+    bool ok = true;
+    if (m_spec.tile_width) {
+        // Handle tile emulation -- output the buffered pixels
+        ASSERT (m_tilebuffer.size());
+        ok &= write_scanlines (m_spec.y, m_spec.y+m_spec.height, 0,
+                               m_spec.format, &m_tilebuffer[0]);
+        std::vector<unsigned char>().swap (m_tilebuffer);
+    }
+
     if (m_png && m_info) {
         PNG_pvt::finish_image (m_png);
         PNG_pvt::destroy_write_struct (m_png, m_info);
     }
-    if (m_file) {
-        fclose (m_file);
-        m_file = NULL;
-    }
-
+    fclose (m_file);
+    m_file = NULL;
     init ();      // re-initialize
-    return true;  // How can we fail?
-                  // Epicly. -- IneQuation
+    return ok;
 }
 
 
@@ -436,7 +457,8 @@ ICOOutput::write_scanline (int y, int z, TypeDesc format,
 {
     m_spec.auto_stride (xstride, format, spec().nchannels);
     const void *origdata = data;
-    data = to_native_scanline (format, data, xstride, m_scratch);
+    data = to_native_scanline (format, data, xstride, m_scratch,
+                               m_dither, y, z);
     if (data == origdata) {
         m_scratch.assign ((unsigned char *)data,
                           (unsigned char *)data + m_spec.scanline_bytes());
@@ -525,5 +547,18 @@ ICOOutput::write_scanline (int y, int z, TypeDesc format,
     return true;
 }
 
+
+
+bool
+ICOOutput::write_tile (int x, int y, int z, TypeDesc format,
+                       const void *data, stride_t xstride,
+                       stride_t ystride, stride_t zstride)
+{
+    // Emulate tiles by buffering the whole image
+    return copy_tile_to_image_buffer (x, y, z, format, data, xstride,
+                                      ystride, zstride, &m_tilebuffer[0]);
+}
+
+
 OIIO_PLUGIN_NAMESPACE_END
 
