@@ -38,14 +38,15 @@
 
 #include <boost/foreach.hpp>
 #include <boost/regex.hpp>
+#include <boost/scoped_array.hpp>
 
-#include "argparse.h"
-#include "strutil.h"
-#include "imageio.h"
-#include "imagebuf.h"
-#include "imagebufalgo.h"
-#include "hash.h"
-#include "filesystem.h"
+#include "OpenImageIO/argparse.h"
+#include "OpenImageIO/strutil.h"
+#include "OpenImageIO/imageio.h"
+#include "OpenImageIO/imagebuf.h"
+#include "OpenImageIO/imagebufalgo.h"
+#include "OpenImageIO/hash.h"
+#include "OpenImageIO/filesystem.h"
 
 OIIO_NAMESPACE_USING;
 
@@ -86,12 +87,12 @@ print_sha1 (ImageInput *input)
             printf ("    SHA-1: unable to compute, image is too big\n");
             return;
         }
-        std::vector<unsigned char> buf((size_t)size);
+        boost::scoped_array<char> buf (new char [size]);
         if (! input->read_image (TypeDesc::UNKNOWN /*native*/, &buf[0])) {
             printf ("    SHA-1: unable to compute, could not read image\n");
             return;
         }
-        sha.appendvec (buf);
+        sha.append (&buf[0], size);
     }
 
     printf ("    SHA-1: %s\n", sha.digest().c_str());
@@ -444,8 +445,8 @@ print_info_subimage (int current_subimage, int max_subimages, ImageSpec &spec,
         int bits = spec.get_int_attribute ("oiio:BitsPerSample", 0);
         printf (", %d channel, %s%s%s", spec.nchannels,
                 spec.deep ? "deep " : "",
-                extended_format_name(spec.format, bits),
-                spec.depth > 1 ? " volume" : "");
+                spec.depth > 1 ? "volume " : "",
+                extended_format_name(spec.format, bits));
         printf (" %s", input->format_name());
         printf ("\n");
     }
@@ -534,9 +535,9 @@ print_info (const std::string &filename, size_t namefieldlength,
                 spec.width, spec.height);
         if (spec.depth > 1)
             printf (" x %4d", spec.depth);
-        printf (", %d channel, ", spec.nchannels);
-        if (spec.deep)
-            printf ("deep ");
+        printf (", %d channel, %s%s", spec.nchannels,
+                spec.deep ? "deep " : "",
+                spec.depth > 1 ? "volume " : "");
         if (spec.channelformats.size()) {
             for (size_t c = 0;  c < spec.channelformats.size();  ++c)
                 printf ("%s%s", c ? "/" : "",
@@ -545,8 +546,6 @@ print_info (const std::string &filename, size_t namefieldlength,
             int bits = spec.get_int_attribute ("oiio:BitsPerSample", 0);
             printf ("%s", extended_format_name(spec.format, bits));
         }
-        if (spec.depth > 1)
-            printf (" volume");
         printf (" %s", input->format_name());
         if (sum) {
             imagesize_t imagebytes = spec.image_bytes (true);
