@@ -46,6 +46,24 @@ extern "C" {
 
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
+
+#define MAX_DATA_BYTES_IN_MARKER 65519L
+#define ICC_HEADER_SIZE 14
+#define ICC_PROFILE_ATTR "ICCProfile"
+
+// Chroma sub-sampling values for jpeg_compress_struct / jpeg_component_info
+#define JPEG_SUBSAMPLING_ATTR "jpeg:subsampling"
+#define JPEG_444_STR "4:4:4"
+#define JPEG_422_STR "4:2:2"
+#define JPEG_420_STR "4:2:0"
+#define JPEG_411_STR "4:1:1"
+
+static const int JPEG_444_COMP[6] = {1,1, 1,1, 1,1};
+static const int JPEG_422_COMP[6] = {2,1, 1,1, 1,1};
+static const int JPEG_420_COMP[6] = {2,2, 1,1, 1,1};
+static const int JPEG_411_COMP[6] = {4,1, 1,1, 1,1};
+
+
 class JpgInput : public ImageInput {
  public:
     JpgInput () { init(); }
@@ -55,9 +73,6 @@ class JpgInput : public ImageInput {
     virtual bool open (const std::string &name, ImageSpec &spec);
     virtual bool open (const std::string &name, ImageSpec &spec,
                        const ImageSpec &config);
-    virtual bool seek_subimage (int index, int miplevel, ImageSpec &newspec) {
-        return (index == 0 && miplevel == 0);   // JPEG has only one subimage
-    }
     virtual bool read_native_scanline (int y, int z, void *data);
     virtual bool close ();
     const std::string &filename () const { return m_filename; }
