@@ -33,6 +33,7 @@
 
 #include <boost/bind.hpp>
 #include <boost/shared_ptr.hpp>
+#include <boost/regex.hpp>
 
 #include <OpenEXR/half.h>
 
@@ -41,14 +42,14 @@
 #include <limits>
 #include <stdexcept>
 
-#include "imagebuf.h"
-#include "imagebufalgo.h"
-#include "imagebufalgo_util.h"
-#include "dassert.h"
-#include "sysutil.h"
-#include "filter.h"
-#include "thread.h"
-#include "filesystem.h"
+#include "OpenImageIO/imagebuf.h"
+#include "OpenImageIO/imagebufalgo.h"
+#include "OpenImageIO/imagebufalgo_util.h"
+#include "OpenImageIO/dassert.h"
+#include "OpenImageIO/sysutil.h"
+#include "OpenImageIO/filter.h"
+#include "OpenImageIO/thread.h"
+#include "OpenImageIO/filesystem.h"
 #include "kissfft.hh"
 
 #ifdef USE_FREETYPE
@@ -175,6 +176,20 @@ ImageBufAlgo::IBAprep (ROI &roi, ImageBuf *dst,
             set_roi_full (spec, full_roi);
         else
             set_roi_full (spec, roi);
+
+        if (prepflags & IBAprep_NO_COPY_METADATA)
+            spec.extra_attribs.clear();
+        else if (! (prepflags & IBAprep_COPY_ALL_METADATA)) {
+            // Since we're altering pixels, be sure that any existing SHA
+            // hash of dst's pixel values is erased.
+            spec.erase_attribute ("oiio:SHA-1");
+            static boost::regex regex_sha ("SHA-1=[[:xdigit:]]*[ ]*");
+            std::string desc = spec.get_string_attribute ("ImageDescription");
+            if (desc.size())
+                spec.attribute ("ImageDescription",
+                                boost::regex_replace (desc, regex_sha, ""));
+        }
+
         dst->alloc (spec);
     }
     if (prepflags & IBAprep_REQUIRE_ALPHA) {
@@ -351,14 +366,15 @@ resize_ (ImageBuf &dst, const ImageBuf &src,
     // will filter the source over [x-radi, x+radi] X [y-radj,y+radj].
     int radi = (int) ceilf (filterrad/xratio);
     int radj = (int) ceilf (filterrad/yratio);
-
+    int xtaps = 2*radi + 1;
+    int ytaps = 2*radj + 1;
     bool separable = filter->separable();
-    float *column = NULL;
+    float *xfiltval = NULL, *yfiltval = NULL;
     if (separable) {
-        // Allocate one column for the first horizontal filter pass
-        column = ALLOCA (float, (2 * radj + 1) * nchannels);
+        // Allocate temp space to cache the filter weights
+        xfiltval = ALLOCA (float, xtaps);
+        yfiltval = ALLOCA (float, ytaps);
     }
-
 #if 0
     std::cerr << "Resizing " << srcspec.full_width << "x" << srcspec.full_height
               << " to " << dstspec.full_width << "x" << dstspec.full_height << "\n";
@@ -387,6 +403,21 @@ resize_ (ImageBuf &dst, const ImageBuf &src,
         float src_yf = srcfy + t * srcfh;
         int src_y;
         float src_yf_frac = floorfrac (src_yf, &src_y);
+
+        // If using separable filters, our vertical set of filter tap
+        // weights will be the same for the whole scanline we're on.  Just
+        // compute and normalize them once.
+        float totalweight_y = 0.0f;
+        if (separable) {
+            for (int j = 0;  j < ytaps;  ++j) {
+                float w = filter->yfilt (yratio * (j-radj-(src_yf_frac-0.5f)));
+                yfiltval[j] = w;
+                totalweight_y += w;
+            }
+            for (int i = 0;  i < ytaps;  ++i)
+                yfiltval[i] /= totalweight_y;
+        }
+
         for (int x = roi.xbegin;  x < roi.xend;  ++x) {
             float s = (x-dstfx+0.5f)*dstpixelwidth;
             float src_xf = srcfx + s * srcfw;
@@ -394,41 +425,50 @@ resize_ (ImageBuf &dst, const ImageBuf &src,
             float src_xf_frac = floorfrac (src_xf, &src_x);
             for (int c = 0;  c < nchannels;  ++c)
                 pel[c] = 0.0f;
-            float totalweight = 0.0f;
             if (separable) {
-                // First, filter horizontally
-                memset (column, 0, (2*radj+1)*nchannels*sizeof(float));
-                float *p = column;
-                for (int j = -radj;  j <= radj;  ++j, p += nchannels) {
-                    totalweight = 0.0f;
-                    int yy = src_y+j;
-                    ImageBuf::ConstIterator<SRCTYPE> srcpel (src, src_x-radi, src_x+radi+1,
-                                                             yy, yy+1, 0, 1, ImageBuf::WrapClamp);
-                    for (int i = -radi;  i <= radi;  ++i, ++srcpel) {
-                        float w = filter->xfilt (xratio * (i-(src_xf_frac-0.5f)));
-                        totalweight += w;
-                        if (w != 0.0f) {
-                            for (int c = 0;  c < nchannels;  ++c)
-                                p[c] += w * srcpel[c];
-                        }
-                    }
-                    if (totalweight != 0.0f) {
-                        for (int c = 0;  c < nchannels;  ++c)
-                            p[c] /= totalweight;
-                    }
-                }
-                // Now filter vertically
-                totalweight = 0.0f;
-                p = column;
-                for (int j = -radj;  j <= radj;  ++j, p += nchannels) {
-                    float w = filter->yfilt (yratio * (j-(src_yf_frac-0.5f)));
-                    totalweight += w;
-                    for (int c = 0;  c < nchannels;  ++c)
-                        pel[c] += w * p[c];
+                // Cache and normalize the horizontal filter tap weights
+                // just once for this (x,y) position, reuse for all vertical
+                // taps.
+                float totalweight_x = 0.0f;
+                for (int i = 0;  i < xtaps;  ++i) {
+                    float w = filter->xfilt (xratio * (i-radi-(src_xf_frac-0.5f)));
+                    xfiltval[i] = w;
+                    totalweight_x += w;
                 }
 
+                if (totalweight_x != 0.0f) {
+                    for (int i = 0;  i < xtaps;  ++i)  // normalize x filter
+                        xfiltval[i] /= totalweight_x;  // weights
+                    ImageBuf::ConstIterator<SRCTYPE> srcpel (src, src_x-radi, src_x+radi+1,
+                                                             src_y-radj, src_y+radj+1,
+                                                             0, 1, ImageBuf::WrapClamp);
+                    for (int j = -radj;  j <= radj;  ++j) {
+                        float wy = yfiltval[j+radj];
+                        if (wy == 0.0f) {
+                            // 0 weight for this y tap -- move to next line
+                            srcpel.pos (srcpel.x(), srcpel.y()+1, srcpel.z());
+                            continue;
+                        }
+                        for (int i = 0;  i < xtaps; ++i, ++srcpel) {
+                            float w = wy * xfiltval[i];
+                            for (int c = 0;  c < nchannels;  ++c)
+                                pel[c] += w * srcpel[c];
+                        }
+                    }
+                }
+                // Copy the pixel value (already normalized) to the output.
+                DASSERT (out.x() == x && out.y() == y);
+                if (totalweight_y == 0.0f) {
+                    // zero it out
+                    for (int c = 0;  c < nchannels;  ++c)
+                        out[c] = 0.0f;
+                } else {
+                    for (int c = 0;  c < nchannels;  ++c)
+                        out[c] = pel[c];
+                }
             } else {
                 // Non-separable
+                float totalweight = 0.0f;
                 ImageBuf::ConstIterator<SRCTYPE> srcpel (src, src_x-radi, src_x+radi+1,
                                                        src_y-radi, src_y+radi+1,
                                                        0, 1, ImageBuf::WrapClamp);
@@ -445,19 +485,19 @@ resize_ (ImageBuf &dst, const ImageBuf &src,
                     }
                 }
                 DASSERT (srcpel.done());
+                // Rescale pel to normalize the filter and write it to the
+                // output image.
+                DASSERT (out.x() == x && out.y() == y);
+                if (totalweight == 0.0f) {
+                    // zero it out
+                    for (int c = 0;  c < nchannels;  ++c)
+                        out[c] = 0.0f;
+                } else {
+                    for (int c = 0;  c < nchannels;  ++c)
+                        out[c] = pel[c] / totalweight;
+                }
             }
 
-            // Rescale pel to normalize the filter, then write it to the
-            // image.
-            DASSERT (out.x() == x && out.y() == y);
-            if (totalweight == 0.0f) {
-                // zero it out
-                for (int c = 0;  c < nchannels;  ++c)
-                    out[c] = 0.0f;
-            } else {
-                for (int c = 0;  c < nchannels;  ++c)
-                    out[c] = pel[c] / totalweight;
-            }
             ++out;
         }
     }
@@ -1053,6 +1093,107 @@ ImageBufAlgo::ifft (ImageBuf &dst, const ImageBuf &src,
 
 
 
+template<class Rtype, class Atype>
+static bool
+polar_to_complex_impl (ImageBuf &R, const ImageBuf &A, ROI roi, int nthreads)
+{
+    if (nthreads != 1 && roi.npixels() >= 1000) {
+        // Possible multiple thread case -- recurse via parallel_image
+        ImageBufAlgo::parallel_image (
+            boost::bind(polar_to_complex_impl<Rtype,Atype>, boost::ref(R), boost::cref(A),
+                        _1 /*roi*/, 1 /*nthreads*/),
+            roi, nthreads);
+        return true;
+    }
+
+    ImageBuf::ConstIterator<Atype> a (A, roi);
+    for (ImageBuf::Iterator<Rtype> r (R, roi);  !r.done();  ++r, ++a) {
+        float amp = a[0];
+        float phase = a[1];
+        float sine, cosine;
+        sincos (phase, &sine, &cosine);
+        r[0] = amp * cosine;
+        r[1] = amp * sine;
+    }
+    return true;
+}
+
+
+
+template<class Rtype, class Atype>
+static bool
+complex_to_polar_impl (ImageBuf &R, const ImageBuf &A, ROI roi, int nthreads)
+{
+    if (nthreads != 1 && roi.npixels() >= 1000) {
+        // Possible multiple thread case -- recurse via parallel_image
+        ImageBufAlgo::parallel_image (
+            boost::bind(complex_to_polar_impl<Rtype,Atype>, boost::ref(R), boost::cref(A),
+                        _1 /*roi*/, 1 /*nthreads*/),
+            roi, nthreads);
+        return true;
+    }
+
+    ImageBuf::ConstIterator<Atype> a (A, roi);
+    for (ImageBuf::Iterator<Rtype> r (R, roi);  !r.done();  ++r, ++a) {
+        float real = a[0];
+        float imag = a[1];
+        float phase = std::atan2 (imag, real);
+        if (phase < 0.0f)
+            phase += float (M_TWO_PI);
+        r[0] = hypotf (real, imag);
+        r[1] = phase;
+    }
+    return true;
+}
+
+
+
+bool
+ImageBufAlgo::polar_to_complex (ImageBuf &dst, const ImageBuf &src,
+                             ROI roi, int nthreads)
+{
+    if (src.nchannels() != 2) {
+        dst.error ("polar_to_complex can only be done on 2-channel");
+        return false;
+    }
+
+    if (! IBAprep (roi, &dst, &src))
+        return false;
+    if (dst.nchannels() != 2) {
+        dst.error ("polar_to_complex can only be done on 2-channel");
+        return false;
+    }
+    OIIO_DISPATCH_COMMON_TYPES2 ("polar_to_complex", polar_to_complex_impl, dst.spec().format,
+                                 src.spec().format, dst, src, roi, nthreads);
+    return true;
+}
+
+
+
+
+bool
+ImageBufAlgo::complex_to_polar (ImageBuf &dst, const ImageBuf &src,
+                             ROI roi, int nthreads)
+{
+    if (src.nchannels() != 2) {
+        dst.error ("complex_to_polar can only be done on 2-channel");
+        return false;
+    }
+
+    if (! IBAprep (roi, &dst, &src))
+        return false;
+    if (dst.nchannels() != 2) {
+        dst.error ("complex_to_polar can only be done on 2-channel");
+        return false;
+    }
+    OIIO_DISPATCH_COMMON_TYPES2 ("complex_to_polar", complex_to_polar_impl, dst.spec().format,
+                                 src.spec().format, dst, src, roi, nthreads);
+    return true;
+}
+
+
+
+
 #ifdef USE_FREETYPE
 namespace { // anon
 static mutex ft_mutex;
@@ -1134,6 +1275,11 @@ ImageBufAlgo::render_text (ImageBuf &R, int x, int y, const std::string &text,
             font = f;
     }
 
+    if (font.empty() || ! Filesystem::is_regular (font)) {
+        R.error ("Could not find font \"%s\"", font);
+        return false;
+    }
+
     FT_Face face;      // handle to face object
     error = FT_New_Face (ft_library, font.c_str(), 0 /* face index */, &face);
     if (error) {
