@@ -40,10 +40,10 @@
 #include <cmath>
 #include <iostream>
 
-#include "imagebuf.h"
-#include "imagebufalgo.h"
-#include "imagebufalgo_util.h"
-#include "thread.h"
+#include "OpenImageIO/imagebuf.h"
+#include "OpenImageIO/imagebufalgo.h"
+#include "OpenImageIO/imagebufalgo_util.h"
+#include "OpenImageIO/thread.h"
 
 
 
@@ -145,6 +145,26 @@ ImageBufAlgo::crop (ImageBuf &dst, const ImageBuf &src,
 
 
 
+bool 
+ImageBufAlgo::cut (ImageBuf &dst, const ImageBuf &src,
+                   ROI roi, int nthreads)
+{
+    bool ok = crop (dst, src, roi, nthreads);
+    ASSERT(ok);
+    if (! ok)
+        return false;
+    // Crop did the heavy lifting of copying the roi of pixels from src to
+    // dst, but now we need to make it look like we cut that rectangle out
+    // and repositioned it at the origin.
+    dst.specmod().x = 0;
+    dst.specmod().y = 0;
+    dst.specmod().z = 0;
+    dst.set_roi_full (dst.roi());
+    return true;
+}
+
+
+
 template<class D, class S>
 static bool
 flip_ (ImageBuf &dst, const ImageBuf &src, ROI roi, int nthreads)
@@ -298,9 +318,9 @@ circular_shift_ (ImageBuf &dst, const ImageBuf &src,
     ImageBuf::ConstIterator<SRCTYPE,DSTTYPE> s (src, roi);
     ImageBuf::Iterator<DSTTYPE,DSTTYPE> d (dst);
     for (  ;  ! s.done();  ++s) {
-        int dx = dstroi.xbegin + ((s.x() - dstroi.xbegin + xshift) % width);
-        int dy = dstroi.ybegin + ((s.y() - dstroi.ybegin + yshift) % height);
-        int dz = dstroi.zbegin + ((s.z() - dstroi.zbegin + zshift) % depth);
+        int dx = s.x() + xshift;  OIIO::wrap_periodic (dx, dstroi.xbegin, width);
+        int dy = s.y() + yshift;  OIIO::wrap_periodic (dy, dstroi.ybegin, height);
+        int dz = s.z() + zshift;  OIIO::wrap_periodic (dz, dstroi.zbegin, depth);
         d.pos (dx, dy, dz);
         if (! d.exists())
             continue;
