@@ -44,19 +44,19 @@
 #include <OpenEXR/ImathMatrix.h>
 #include <OpenEXR/half.h>
 
-#include "argparse.h"
-#include "dassert.h"
-#include "filesystem.h"
-#include "fmath.h"
-#include "strutil.h"
-#include "sysutil.h"
-#include "timer.h"
-#include "imageio.h"
-#include "imagebuf.h"
-#include "imagebufalgo.h"
-#include "imagebufalgo_util.h"
-#include "thread.h"
-#include "filter.h"
+#include "OpenImageIO/argparse.h"
+#include "OpenImageIO/dassert.h"
+#include "OpenImageIO/filesystem.h"
+#include "OpenImageIO/fmath.h"
+#include "OpenImageIO/strutil.h"
+#include "OpenImageIO/sysutil.h"
+#include "OpenImageIO/timer.h"
+#include "OpenImageIO/imageio.h"
+#include "OpenImageIO/imagebuf.h"
+#include "OpenImageIO/imagebufalgo.h"
+#include "OpenImageIO/imagebufalgo_util.h"
+#include "OpenImageIO/thread.h"
+#include "OpenImageIO/filter.h"
 
 OIIO_NAMESPACE_USING
 
@@ -264,7 +264,7 @@ resize_block_ (ImageBuf &dst, const ImageBuf &src, ROI roi, bool envlatlmode)
                         srcspec.z+srcspec.depth < srcspec.full_z+srcspec.full_depth);
 
     const ImageSpec &dstspec (dst.spec());
-    float *pel = (float *) alloca (dstspec.pixel_bytes());
+    float *pel = ALLOCA (float, dstspec.nchannels);
     float xoffset = (float) dstspec.full_x;
     float yoffset = (float) dstspec.full_y;
     float xscale = 1.0f / (float)dstspec.full_width;
@@ -385,7 +385,7 @@ check_nan_block (const ImageBuf &src, ROI roi, int &found_nonfinite)
 {
     int x0 = roi.xbegin, x1 = roi.xend, y0 = roi.ybegin, y1 = roi.yend;
     const ImageSpec &spec (src.spec());
-    float *pel = (float *) alloca (spec.pixel_bytes());
+    float *pel = ALLOCA (float, spec.nchannels);
     for (int y = y0;  y < y1;  ++y) {
         for (int x = x0;  x < x1;  ++x) {
             src.getpixel (x, y, pel);
@@ -1114,7 +1114,7 @@ make_texture_impl (ImageBufAlgo::MakeTextureMode mode,
         dstspec.attribute ("textureformat", "LatLong Environment");
         configspec.attribute ("wrapmodes", "periodic,clamp");
         if (prman_metadata)
-            dstspec.attribute ("PixarTextureFormat", "Latlong Environment");
+            dstspec.attribute ("PixarTextureFormat", "LatLong Environment");
     } else {
         dstspec.attribute ("textureformat", "Plain Texture");
         if (prman_metadata)
