@@ -36,19 +36,18 @@
 
 #include <OpenEXR/ImathMatrix.h>
 
-#include "dassert.h"
-#include "typedesc.h"
-#include "varyingref.h"
-#include "ustring.h"
-#include "strutil.h"
-#include "thread.h"
-#include "fmath.h"
-#include "filter.h"
-#include "imageio.h"
+#include "OpenImageIO/dassert.h"
+#include "OpenImageIO/typedesc.h"
+#include "OpenImageIO/varyingref.h"
+#include "OpenImageIO/ustring.h"
+#include "OpenImageIO/strutil.h"
+#include "OpenImageIO/thread.h"
+#include "OpenImageIO/fmath.h"
+#include "OpenImageIO/filter.h"
+#include "OpenImageIO/imageio.h"
+#include "OpenImageIO/texture.h"
+#include "OpenImageIO/imagecache.h"
 
-#include "texture.h"
-
-#include "imagecache.h"
 #include "imagecache_pvt.h"
 #include "texture_pvt.h"
 #include "../field3d.imageio/field3d_backdoor.h"
@@ -217,8 +216,8 @@ bool
 TextureSystemImpl::texture3d_lookup_nomip (TextureFile &texturefile,
                             PerThreadInfo *thread_info, 
                             TextureOpt &options,
-                            const Imath::V3f &_P, const Imath::V3f &_dPdx,
-                            const Imath::V3f &_dPdy, const Imath::V3f &_dPdz,
+                            const Imath::V3f &P, const Imath::V3f &dPdx,
+                            const Imath::V3f &dPdy, const Imath::V3f &dPdz,
                             float *result)
 {
     // Initialize results to 0.  We'll add from here on as we sample.
@@ -250,7 +249,7 @@ TextureSystemImpl::texture3d_lookup_nomip (TextureFile &texturefile,
         &TextureSystemImpl::accum3d_sample_bilinear,
     };
     accum3d_prototype accumer = accum_functions[(int)options.interpmode];
-    bool ok = (this->*accumer) (_P, 0, texturefile, thread_info, options,
+    bool ok = (this->*accumer) (P, 0, texturefile, thread_info, options,
                                 1.0f, result, dresultds, dresultdt, dresultdr);
 
     // Update stats
