@@ -36,7 +36,7 @@
 #ifndef OPENIMAGEIO_TEXTURE_PVT_H
 #define OPENIMAGEIO_TEXTURE_PVT_H
 
-#include "version.h"
+#include "OpenImageIO/texture.h"
 
 
 OIIO_NAMESPACE_ENTER
@@ -67,43 +67,40 @@ public:
     TextureSystemImpl (ImageCache *imagecache);
     virtual ~TextureSystemImpl ();
 
-    virtual bool attribute (const std::string &name, TypeDesc type, const void *val);
-    virtual bool attribute (const std::string &name, int val) {
+    virtual bool attribute (string_view name, TypeDesc type, const void *val);
+    virtual bool attribute (string_view name, int val) {
         return attribute (name, TypeDesc::INT, &val);
     }
-    virtual bool attribute (const std::string &name, float val) {
+    virtual bool attribute (string_view name, float val) {
         return attribute (name, TypeDesc::FLOAT, &val);
     }
-    virtual bool attribute (const std::string &name, double val) {
+    virtual bool attribute (string_view name, double val) {
         float f = (float) val;
         return attribute (name, TypeDesc::FLOAT, &f);
     }
-    virtual bool attribute (const std::string &name, const char *val) {
-        return attribute (name, TypeDesc::STRING, &val);
-    }
-    virtual bool attribute (const std::string &name, const std::string &val) {
+    virtual bool attribute (string_view name, string_view val) {
         const char *s = val.c_str();
         return attribute (name, TypeDesc::STRING, &s);
     }
 
-    virtual bool getattribute (const std::string &name, TypeDesc type, void *val);
-    virtual bool getattribute (const std::string &name, int &val) {
+    virtual bool getattribute (string_view name, TypeDesc type, void *val);
+    virtual bool getattribute (string_view name, int &val) {
         return getattribute (name, TypeDesc::INT, &val);
     }
-    virtual bool getattribute (const std::string &name, float &val) {
+    virtual bool getattribute (string_view name, float &val) {
         return getattribute (name, TypeDesc::FLOAT, &val);
     }
-    virtual bool getattribute (const std::string &name, double &val) {
+    virtual bool getattribute (string_view name, double &val) {
         float f;
         bool ok = getattribute (name, TypeDesc::FLOAT, &f);
         if (ok)
             val = f;
         return ok;
     }
-    virtual bool getattribute (const std::string &name, char **val) {
+    virtual bool getattribute (string_view name, char **val) {
         return getattribute (name, TypeDesc::STRING, val);
     }
-    virtual bool getattribute (const std::string &name, std::string &val) {
+    virtual bool getattribute (string_view name, std::string &val) {
         const char *s;
         bool ok = getattribute (name, TypeDesc::STRING, &s);
         if (ok)
@@ -361,14 +358,14 @@ private:
     typedef bool (TextureSystemImpl::*texture3d_lookup_prototype)
             (TextureFile &texfile, PerThreadInfo *thread_info,
              TextureOpt &options,
-             const Imath::V3f &_P, const Imath::V3f &_dPdx,
-             const Imath::V3f &_dPdy, const Imath::V3f &_dPdz,
+             const Imath::V3f &P, const Imath::V3f &dPdx,
+             const Imath::V3f &dPdy, const Imath::V3f &dPdz,
              float *result);
     bool texture3d_lookup_nomip (TextureFile &texfile,
                                  PerThreadInfo *thread_info, 
                                  TextureOpt &options,
-                                 const Imath::V3f &_P, const Imath::V3f &_dPdx,
-                                 const Imath::V3f &_dPdy, const Imath::V3f &_dPdz,
+                                 const Imath::V3f &P, const Imath::V3f &dPdx,
+                                 const Imath::V3f &dPdy, const Imath::V3f &dPdz,
                                  float *result);
     typedef bool (TextureSystemImpl::*accum3d_prototype)
                         (const Imath::V3f &P, int level,
