@@ -37,26 +37,24 @@
 
 #include <OpenEXR/ImathMatrix.h>
 
-#include "dassert.h"
-#include "typedesc.h"
-#include "varyingref.h"
-#include "ustring.h"
-#include "strutil.h"
-#include "sysutil.h"
-#include "thread.h"
-#include "fmath.h"
-#include "filter.h"
-#include "optparser.h"
-#include "imageio.h"
-
-#include "texture.h"
-
-#include "imagecache.h"
+#include "OpenImageIO/dassert.h"
+#include "OpenImageIO/typedesc.h"
+#include "OpenImageIO/varyingref.h"
+#include "OpenImageIO/ustring.h"
+#include "OpenImageIO/strutil.h"
+#include "OpenImageIO/sysutil.h"
+#include "OpenImageIO/thread.h"
+#include "OpenImageIO/fmath.h"
+#include "OpenImageIO/filter.h"
+#include "OpenImageIO/optparser.h"
+#include "OpenImageIO/imageio.h"
+#include "OpenImageIO/texture.h"
+#include "OpenImageIO/imagecache.h"
+#include "OpenImageIO/imagebuf.h"
+#include "OpenImageIO/imagebufalgo.h"
+#include <boost/random.hpp>
 #include "imagecache_pvt.h"
 #include "texture_pvt.h"
-#include "imagebuf.h"
-#include "imagebufalgo.h"
-#include <boost/random.hpp>
 
 
 OIIO_NAMESPACE_ENTER
@@ -271,7 +269,7 @@ TextureSystemImpl::reset_stats ()
 
 
 bool
-TextureSystemImpl::attribute (const std::string &name, TypeDesc type,
+TextureSystemImpl::attribute (string_view name, TypeDesc type,
                               const void *val)
 {
     if (name == "options" && type == TypeDesc::STRING) {
@@ -306,7 +304,7 @@ TextureSystemImpl::attribute (const std::string &name, TypeDesc type,
 
 
 bool
-TextureSystemImpl::getattribute (const std::string &name, TypeDesc type,
+TextureSystemImpl::getattribute (string_view name, TypeDesc type,
                                  void *val)
 {
     if (name == "worldtocommon" && (type == TypeDesc::TypeMatrix ||
