@@ -59,7 +59,7 @@
 #endif
 
 #ifdef _WIN32
-# include "osdep.h"
+# include "OpenImageIO/osdep.h"
 # include <Psapi.h>
 #else
 # include <sys/resource.h>
@@ -70,9 +70,8 @@
 # include <sys/ioctl.h>
 #endif
 
-#include "dassert.h"
-
-#include "sysutil.h"
+#include "OpenImageIO/dassert.h"
+#include "OpenImageIO/sysutil.h"
 
 OIIO_NAMESPACE_ENTER
 {
@@ -245,7 +244,7 @@ Sysutil::this_program_path ()
     size_t cb = sizeof(filename);
     int r=1;
     sysctl(mib, 4, filename, &cb, NULL, 0);
-#elif defined(__GNU__)
+#elif defined(__GNU__) || defined(__OpenBSD__)
     int r = 0;
 #else
     // No idea what platform this is
