@@ -36,9 +36,9 @@
 
 #include "png_pvt.h"
 
-#include "dassert.h"
-#include "imageio.h"
-#include "strutil.h"
+#include "OpenImageIO/dassert.h"
+#include "OpenImageIO/imageio.h"
+#include "OpenImageIO/strutil.h"
 
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
@@ -57,17 +57,22 @@ public:
     virtual bool close ();
     virtual bool write_scanline (int y, int z, TypeDesc format,
                                  const void *data, stride_t xstride);
+    virtual bool write_tile (int x, int y, int z, TypeDesc format,
+                             const void *data, stride_t xstride,
+                             stride_t ystride, stride_t zstride);
 
 private:
     std::string m_filename;           ///< Stash the filename
     FILE *m_file;                     ///< Open image handle
     png_structp m_png;                ///< PNG read structure pointer
     png_infop m_info;                 ///< PNG image info structure pointer
+    unsigned int m_dither;
     int m_color_type;                 ///< PNG color model type
     bool m_convert_alpha;             ///< Do we deassociate alpha?
     float m_gamma;                    ///< Gamma to use for alpha conversion
     std::vector<unsigned char> m_scratch;
     std::vector<png_text> m_pngtext;
+    std::vector<unsigned char> m_tilebuffer;
 
     // Initialize private members to pre-opened state
     void init (void) {
@@ -174,10 +179,20 @@ PNGOutput::open (const std::string &name, const ImageSpec &userspec,
         png_set_compression_strategy(m_png, Z_DEFAULT_STRATEGY);
     }
 
-
     PNG_pvt::write_info (m_png, m_info, m_color_type, m_spec, m_pngtext,
                          m_convert_alpha, m_gamma);
 
+    m_dither = (m_spec.format == TypeDesc::UINT8) ?
+                    m_spec.get_int_attribute ("oiio:dither", 0) : 0;
+
+    m_convert_alpha = m_spec.alpha_channel != -1 &&
+                      !m_spec.get_int_attribute("oiio:UnassociatedAlpha", 0);
+
+    // If user asked for tiles -- which this format doesn't support, emulate
+    // it by buffering the whole image.
+    if (m_spec.tile_width && m_spec.tile_height)
+        m_tilebuffer.resize (m_spec.image_bytes());
+
     return true;
 }
 
@@ -186,17 +201,29 @@ PNGOutput::open (const std::string &name, const ImageSpec &userspec,
 bool
 PNGOutput::close ()
 {
+    if (! m_file) {   // already closed
+        init ();
+        return true;
+    }
+
+    bool ok = true;
+    if (m_spec.tile_width) {
+        // Handle tile emulation -- output the buffered pixels
+        ASSERT (m_tilebuffer.size());
+        ok &= write_scanlines (m_spec.y, m_spec.y+m_spec.height, 0,
+                               m_spec.format, &m_tilebuffer[0]);
+        std::vector<unsigned char>().swap (m_tilebuffer);
+    }
+
     if (m_png)
         PNG_pvt::finish_image (m_png);
     PNG_pvt::destroy_write_struct (m_png, m_info);
 
-    if (m_file) {
-        fclose (m_file);
-        m_file = NULL;
-    }
+    fclose (m_file);
+    m_file = NULL;
 
     init ();      // re-initialize
-    return true;  // How can we fail?
+    return ok;
 }
 
 
@@ -239,7 +266,8 @@ PNGOutput::write_scanline (int y, int z, TypeDesc format,
     y -= m_spec.y;
     m_spec.auto_stride (xstride, format, spec().nchannels);
     const void *origdata = data;
-    data = to_native_scanline (format, data, xstride, m_scratch);
+    data = to_native_scanline (format, data, xstride, m_scratch,
+                               m_dither, y, z);
     if (data == origdata) {
         m_scratch.assign ((unsigned char *)data,
                           (unsigned char *)data+m_spec.scanline_bytes());
@@ -270,5 +298,18 @@ PNGOutput::write_scanline (int y, int z, TypeDesc format,
     return true;
 }
 
+
+
+bool
+PNGOutput::write_tile (int x, int y, int z, TypeDesc format,
+                       const void *data, stride_t xstride,
+                       stride_t ystride, stride_t zstride)
+{
+    // Emulate tiles by buffering the whole image
+    return copy_tile_to_image_buffer (x, y, z, format, data, xstride,
+                                      ystride, zstride, &m_tilebuffer[0]);
+}
+
+
 OIIO_PLUGIN_NAMESPACE_END
 
