@@ -30,8 +30,8 @@
 
 #include <fstream>
 
-#include "filesystem.h"
-#include "imageio.h"
+#include "OpenImageIO/filesystem.h"
+#include "OpenImageIO/imageio.h"
 
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
@@ -49,12 +49,17 @@ public:
     virtual bool close ();
     virtual bool write_scanline (int y, int z, TypeDesc format,
                                  const void *data, stride_t xstride);
+    virtual bool write_tile (int x, int y, int z, TypeDesc format,
+                             const void *data, stride_t xstride,
+                             stride_t ystride, stride_t zstride);
 
 private:
     std::string m_filename;           ///< Stash the filename
     std::ofstream m_file;
     unsigned int m_max_val, m_pnm_type;
+    unsigned int m_dither;
     std::vector<unsigned char> m_scratch;
+    std::vector<unsigned char> m_tilebuffer;
 };
 
 
@@ -167,6 +172,8 @@ PNMOutput::open (const std::string &name, const ImageSpec &userspec,
     m_spec = userspec;  // Stash the spec
     m_spec.set_format (TypeDesc::UINT8);  // Force 8 bit output
     int bits_per_sample = m_spec.get_int_attribute ("oiio:BitsPerSample", 8);
+    m_dither = (m_spec.format == TypeDesc::UINT8) ?
+                    m_spec.get_int_attribute ("oiio:dither", 0) : 0;
 
     if (bits_per_sample == 1)
         m_pnm_type = 4;
@@ -192,6 +199,11 @@ PNMOutput::open (const std::string &name, const ImageSpec &userspec,
     if (m_pnm_type != 1 && m_pnm_type != 4)  // only non-monochrome 
         m_file << m_max_val << std::endl;
 
+    // If user asked for tiles -- which this format doesn't support, emulate
+    // it by buffering the whole image.
+    if (m_spec.tile_width && m_spec.tile_height)
+        m_tilebuffer.resize (m_spec.image_bytes());
+
     return m_file.good();
 }
 
@@ -200,8 +212,20 @@ PNMOutput::open (const std::string &name, const ImageSpec &userspec,
 bool
 PNMOutput::close ()
 {
-    if (m_file.is_open())
-        m_file.close();
+    if (! m_file.is_open()) {   // already closed
+        return true;
+    }
+
+    bool ok = true;
+    if (m_spec.tile_width) {
+        // Handle tile emulation -- output the buffered pixels
+        ASSERT (m_tilebuffer.size());
+        ok &= write_scanlines (m_spec.y, m_spec.y+m_spec.height, 0,
+                               m_spec.format, &m_tilebuffer[0]);
+        std::vector<unsigned char>().swap (m_tilebuffer);
+    }
+
+    m_file.close();
     return true;  
 }
 
@@ -218,7 +242,8 @@ PNMOutput::write_scanline (int y, int z, TypeDesc format,
 
     m_spec.auto_stride (xstride, format, spec().nchannels);
     const void *origdata = data;
-    data = to_native_scanline (format, data, xstride, m_scratch);
+    data = to_native_scanline (format, data, xstride, m_scratch,
+                               m_dither, y, z);
     if (data != origdata) // a conversion happened...
         xstride = spec().nchannels;
 
@@ -250,5 +275,18 @@ PNMOutput::write_scanline (int y, int z, TypeDesc format,
     return m_file.good();
 }
 
+
+
+bool
+PNMOutput::write_tile (int x, int y, int z, TypeDesc format,
+                       const void *data, stride_t xstride,
+                       stride_t ystride, stride_t zstride)
+{
+    // Emulate tiles by buffering the whole image
+    return copy_tile_to_image_buffer (x, y, z, format, data, xstride,
+                                      ystride, zstride, &m_tilebuffer[0]);
+}
+
+
 OIIO_PLUGIN_NAMESPACE_END
 
