@@ -389,7 +389,7 @@ private:
         // For debugging, numeric_cast will throw if precision is lost:
         // value = boost::numeric_cast<TVariable>(buffer);
         value = buffer;
-        return m_file;
+        return m_file.good();
     }
 
     int read_pascal_string (std::string &s, uint16_t mod_padding);
