@@ -28,7 +28,7 @@
   (This is the Modified BSD License)
 */
 
-#include "imagebufalgo.h"
+#include "OpenImageIO/imagebufalgo.h"
 #include "py_oiio.h"
 
 
@@ -222,6 +222,31 @@ IBA_mul_images (ImageBuf &dst, const ImageBuf &A, const ImageBuf &B,
 
 
 
+bool
+IBA_pow_color (ImageBuf &dst, const ImageBuf &A, tuple values_tuple,
+               ROI roi=ROI::All(), int nthreads=0)
+{
+    std::vector<float> values;
+    py_to_stdvector (values, values_tuple);
+    if (roi.defined())
+        values.resize (roi.nchannels(), 0.0f);
+    else if (A.initialized())
+        values.resize (A.nchannels(), 0.0f);
+    else return false;
+    ASSERT (values.size() > 0);
+    return ImageBufAlgo::pow (dst, A, &values[0], roi, nthreads);
+}
+
+bool
+IBA_pow_float (ImageBuf &dst, const ImageBuf &A, float B,
+               ROI roi=ROI::All(), int nthreads=0)
+{
+    return ImageBufAlgo::pow (dst, A, B, roi, nthreads);
+}
+
+
+
+
 bool
 IBA_clamp (ImageBuf &dst, const ImageBuf &src,
            tuple min_, tuple max_,
@@ -531,6 +556,11 @@ void declare_imagebufalgo()
               arg("roi")=ROI::All(), arg("nthreads")=0))
         .staticmethod("crop")
 
+        .def("cut", &ImageBufAlgo::cut,
+             (arg("dst"), arg("src"),
+              arg("roi")=ROI::All(), arg("nthreads")=0))
+        .staticmethod("cut")
+
         .def("paste", &ImageBufAlgo::paste,
              (arg("dst"), arg("xbegin"), arg("ybegin"), arg("zbegin"),
               arg("chbegin"), arg("src"),
@@ -596,6 +626,14 @@ void declare_imagebufalgo()
               arg("roi")=ROI::All(), arg("nthreads")=0))
         .staticmethod("mul")
 
+        .def("pow", &IBA_pow_float,
+             (arg("dst"), arg("A"), arg("B"),
+              arg("roi")=ROI::All(), arg("nthreads")=0))
+        .def("pow", &IBA_pow_color,
+             (arg("dst"), arg("A"), arg("B"),
+              arg("roi")=ROI::All(), arg("nthreads")=0))
+        .staticmethod("pow")
+
         .def("channel_sum", &IBA_channel_sum,
              (arg("dst"), arg("src"),
               arg("roi")=ROI::All(), arg("nthreads")=0))
@@ -735,6 +773,16 @@ void declare_imagebufalgo()
               arg("roi")=ROI::All(), arg("nthreads")=0))
         .staticmethod("ifft")
 
+        .def("polar_to_complex", &ImageBufAlgo::polar_to_complex,
+             (arg("dst"), arg("src"),
+              arg("roi")=ROI::All(), arg("nthreads")=0))
+        .staticmethod("polar_to_complex")
+
+        .def("complex_to_polar", &ImageBufAlgo::complex_to_polar,
+             (arg("dst"), arg("src"),
+              arg("roi")=ROI::All(), arg("nthreads")=0))
+        .staticmethod("complex_to_polar")
+
         .def("fixNonFinite", &IBA_fixNonFinite,
              (arg("dst"), arg("src"), 
               arg("mode")=ImageBufAlgo::NONFINITE_BOX3,
