@@ -43,18 +43,18 @@
 
 #include <boost/bind.hpp>
 
-#include "argparse.h"
-#include "imageio.h"
-#include "ustring.h"
-#include "imagebuf.h"
-#include "imagebufalgo.h"
-#include "imagebufalgo_util.h"
-#include "texture.h"
-#include "fmath.h"
-#include "filesystem.h"
-#include "sysutil.h"
-#include "strutil.h"
-#include "timer.h"
+#include "OpenImageIO/argparse.h"
+#include "OpenImageIO/imageio.h"
+#include "OpenImageIO/ustring.h"
+#include "OpenImageIO/imagebuf.h"
+#include "OpenImageIO/imagebufalgo.h"
+#include "OpenImageIO/imagebufalgo_util.h"
+#include "OpenImageIO/texture.h"
+#include "OpenImageIO/fmath.h"
+#include "OpenImageIO/filesystem.h"
+#include "OpenImageIO/sysutil.h"
+#include "OpenImageIO/strutil.h"
+#include "OpenImageIO/timer.h"
 #include "../libtexture/imagecache_pvt.h"
 
 OIIO_NAMESPACE_USING
