@@ -78,6 +78,13 @@ command += (oiio_app ("oiiotool")
 command += (oiio_app ("oiiotool")
             + " cmul-input.exr --cadd 0,0.25,-0.25 -o cadd2.exr >> out.txt ;\n")
 
+# Test --cpow val (raise all channels by the same power)
+command += (oiio_app ("oiiotool")
+            + " cmul-input.exr --cpow 2 -o cpow1.exr >> out.txt ;\n")
+# Test --cpow val,val,val... (per-channel powers)
+command += (oiio_app ("oiiotool")
+            + " cmul-input.exr --cpow 2,2,1 -o cpow2.exr >> out.txt ;\n")
+
 # Test --add
 command += (oiio_app ("oiiotool")
             + " --pattern constant:color=.1,.2,.3 64x64+0+0 3 "
@@ -104,6 +111,16 @@ command += (oiio_app ("oiiotool") + " "
             + " --histogram:cumulative=1 256x256 0 "
             + "-o histogram_cumulative.tif >> out.txt ;\n")
 
+# test --crop
+command += (oiio_app("oiiotool")
+            + parent + "/oiio-images/grid.tif "
+            + "--crop 100x400+50+200 -o crop.tif ;\n")
+
+# test --cut
+command += (oiio_app("oiiotool")
+            + parent + "/oiio-images/grid.tif "
+            + "--cut 100x400+50+200 -o cut.tif ;\n")
+
 # test paste
 command += (oiio_app("oiiotool")
             + parent + "/oiio-images/grid.tif "
@@ -217,6 +234,18 @@ command += (oiio_app ("oiiotool")
 command += (oiio_app ("oiiotool")
             + "fft.exr --ifft --ch 0,0,0 -o ifft.exr >> out.txt ;\n")
 
+# test --polar, --unpolar
+# note that fft.exr that we built above is in complex form
+command += oiiotool ("fft.exr --polar -d half -o polar.exr")
+command += oiiotool ("polar.exr --unpolar -d half -o unpolar.exr")
+
+# test labels
+command += (oiio_app("oiiotool") + 
+            " --pattern constant:color=0.5,0.0,0.0 128x128 3 --label R " +
+            " --pattern constant:color=0.0,0.5,0.0 128x128 3 --label G " +
+            " --pattern constant:color=0.5,0.0,0.0 128x128 3 --label B " +
+            " --pop --pop --pop " +
+            " R G --add -d half -o labeladd.exr >> out.txt ;\n")
 
 
 # test sequences
@@ -226,6 +255,9 @@ command += (oiio_app("oiiotool") + " --info "
             + " ".join(["copyA.{0:04}.jpg".format(x) for x in range(1,11)])
             + " >> out.txt ;\n")
 
+# test --no-autopremult on a TGA file thet needs it.
+command += oiiotool ("--no-autopremult src/rgba.tga --ch R,G,B -o rgbfromtga.png")
+
 # To add more tests, just append more lines like the above and also add
 # the new 'feature.tif' (or whatever you call it) to the outputs list,
 # below.
@@ -237,13 +269,14 @@ outputs = [ "filled.tif", "autotrim.tif",
             "resize64.tif", "resize512.tif",
             "fit.tif", "fit2.tif",
             "histogram_regular.tif", "histogram_cumulative.tif",
-            "pasted.tif", "mosaic.tif",
+            "crop.tif", "cut.tif", "pasted.tif", "mosaic.tif",
             "flip.tif", "flop.tif", "flipflop.tif", "transpose.tif",
             "cshift.tif",
             "chanshuffle.tif", "ch-rgba.exr", "ch-z.exr",
             "chappend-rgbaz.exr", "chname.exr", "flat.exr",
             "cmul1.exr", "cmul2.exr",
             "cadd1.exr", "cadd2.exr",
+            "cpow1.exr", "cpow2.exr",
             "add.exr", "sub.exr", "chsum.tif",
             "rgbahalf-zfloat.exr",
             "tahoe-filled.tif",
@@ -253,6 +286,9 @@ outputs = [ "filled.tif", "autotrim.tif",
             "bsplinekernel.exr", "bspline-blur.tif",
             "gauss5x5-blur.tif", "unsharp.tif",
             "fft.exr", "ifft.exr",
+            "polar.exr", "unpolar.exr",
+            "labeladd.exr",
+            "rgbfromtga.png",
             "out.txt" ]
 
 #print "Running this command:\n" + command + "\n"
