@@ -25,14 +25,16 @@ command += "python test_imagebufalgo.py > out.txt ;"
 for f in [ "black.tif", "filled.tif", "checker.tif",
            "chanshuffle.tif", "ch-rgba.exr", "ch-z.exr",
            "chappend-rgbaz.exr", "flat.exr",
-           "crop.tif", "pasted.tif",
+           "crop.tif", "cut.tif", "pasted.tif",
            "flip.tif", "flop.tif", "flipflop.tif", "transpose.tif",
            "cshift.tif", "cadd1.exr", "cadd2.exr", "add.exr", "sub.exr",
-           "cmul1.exr", "cmul2.exr", "chsum.tif", "grid-clamped.tif",
+           "cmul1.exr", "cmul2.exr", "cpow1.exr", "cpow2.exr",
+           "chsum.tif", "grid-clamped.tif",
            "rangecompress.tif", "rangeexpand.tif",
            "resize.tif", "resample.tif",
            "bsplinekernel.exr", "bspline-blur.tif", "unsharp.tif",
-           "fft.exr", "ifft.exr", "tahoe-filled.tif",
+           "fft.exr", "ifft.exr", "polar.exr", "complex.exr",
+           "tahoe-filled.tif",
            "box3.exr",
            "a_over_b.exr",
            "tahoe-small.tx"
